#pragma once

S_BEGIN_DECLS

/**
 * @file
 * @defgroup Console Console
 * @addtogroup Console
 * @{
 * @brief Set of functions and data structures to deal with console or terminal
 *        related thing.
 */

/**
 * Get the height and width of the console.
 *
 * @param height The output parameter for the height of the console.
 * @param width The output parameter for the width of the console.
 *
 * @code

 * @endcode
 */
S_EXPORTED
void
s_console_get_size (sint * height, sint * width);

/**
 * @defgroup SConsoleBuffer SConsoleBuffer
 * @addtogroup SConsoleBuffer
 * @{
 * @brief Datastructure that makes dealing with writing to the console/terminal
 *        in big chunks easyer.
 *
 * SConsoleBuffer internally uses @c suint for the buffer, this so that when
 * we write to the terminal/console we can draw any charercter... Hopefully.
 */


/**
 * @brief An opaque datastructure that reperesents the console buffer object.
 */
typedef struct _SConsoleBuffer SConsoleBuffer;


typedef enum {
  S_CONSOLE_BUFFER_MODE_NONE = 0,
  S_CONSOLE_BUFFER_MODE_LINE,
  S_CONSOLE_BUFFER_MODE_FULL,
  S_CONSOLE_BUFFER_MODE_UNUSED_0,
  S_CONSOLE_BUFFER_MODE_UNUSED_1,
  S_CONSOLE_BUFFER_MODE_UNUSED_2
} SConsoleBufferMode;


/**
 * @brief Create a new SConsoleBuffer.
 *
 * @param height The height of the buffer 
 * @param width The width of the buffer.
 *
 * @returns a new SConsoleBuffer object.
 */
S_EXPORTED
SConsoleBuffer *
s_console_buffer_new (sint height, sint width, SConsoleBufferMode mode);

/**
 * @brief Free a SConsoleBuffer
 * 
 * @param self The object to be freed.
 */
S_EXPORTED
void
s_console_buffer_free (SConsoleBuffer * self);

/**
 *
 */
S_EXPORTED
void
s_console_buffer_set_char (SConsoleBuffer * self, suchar c, sint x, sint y);

S_EXPORTED
void
s_console_buffer_set_mode (SConsoleBuffer * self, SConsoleBufferMode mode);

S_EXPORTED
void
s_console_buffer_draw (SConsoleBuffer * self);

S_EXPORTED
void
s_console_buffer_resize (SConsoleBuffer * self, sint x, sint y);

/**
 * @}
 * @}
 */

S_END_DECLS
